@@ -1,5 +1,5 @@
 /*
-  libspeechd.c - Shared library for easy acces to Speech Dispatcher functions
+  libspeechd.c - Shared library for easy access to Speech Dispatcher functions
  *
  * Copyright (C) 2001, 2002, 2003, 2006, 2007, 2008 Brailcom, o.p.s.
  *
@@ -241,7 +241,7 @@ static void _init_debug(void)
 	if (!spd_debug) {
 		spd_debug = fopen("/tmp/libspeechd.log", "w");
 		if (spd_debug == NULL)
-			SPD_FATAL("COULDN'T ACCES FILE INTENDED FOR DEBUG");
+			SPD_FATAL("COULDN'T ACCESS FILE INTENDED FOR DEBUG");
 
 		SPD_DBG("Debugging started");
 	}
@@ -381,9 +381,11 @@ spawn_server(const SPDConnectionAddress * address, int is_localhost,
 			    g_strdup_printf
 			    ("Autospawn failed. Speech Dispatcher refused to start with error code, "
 			     "stating this as a reason: %s", stderr_output);
+			g_free(stderr_output);
 			return 1;
 		} else {
 			*spawn_error = NULL;
+			g_free(stderr_output);
 			return 0;
 		}
 	}
@@ -618,7 +620,9 @@ void spd_close(SPDConnection * connection)
 	}
 
 	/* close the socket */
-	close(connection->socket);
+	if (connection->stream != NULL)
+		fclose(connection->stream);
+	connection->stream = NULL;
 
 	pthread_mutex_unlock(&connection->ssip_mutex);
 
@@ -718,7 +722,7 @@ int spd_say(SPDConnection * connection, SPDPriority priority, const char *text)
 	return msg_id;
 }
 
-/* The same as spd_say, accepts also formated strings */
+/* The same as spd_say, accepts also formatted strings */
 int
 spd_sayf(SPDConnection * connection, SPDPriority priority, const char *format,
 	 ...)
@@ -880,6 +884,7 @@ spd_wchar(SPDConnection * connection, SPDPriority priority, wchar_t wcharacter)
 	ret = wcrtomb(character, wcharacter, NULL);
 	if (ret <= 0)
 		RET(-1);
+	character[ret] = '\0';
 
 	ret = spd_set_priority(connection, priority);
 	if (ret)
@@ -1453,6 +1458,16 @@ char **spd_list_voices(SPDConnection * connection)
 	return voices;
 }
 
+void free_spd_symbolic_voices(char **voices)
+{
+	int i = 0;
+	while (voices != NULL && voices[i] != NULL) {
+		free(voices[i]);
+		++i;
+	}
+	free(voices);
+}
+
 SPDVoice **spd_list_synthesis_voices(SPDConnection * connection)
 {
 	char **svoices_str;
@@ -1765,21 +1780,35 @@ static int spd_set_priority(SPDConnection * connection, SPDPriority priority)
 	return spd_execute_command_wo_mutex(connection, command);
 }
 
+struct get_reply_data {
+	GString *str;
+	char *line;
+};
+
+static void get_reply_cleanup(void *arg)
+{
+	struct get_reply_data *data = arg;
+	g_string_free(data->str, TRUE);
+	free(data->line);
+}
+
 static char *get_reply(SPDConnection * connection)
 {
-	GString *str;
-	char *line = NULL;
 	size_t N = 0;
 	int bytes;
 	char *reply;
 	gboolean errors = FALSE;
+	struct get_reply_data data;
 
-	str = g_string_new("");
+	data.line = NULL;
+	data.str = g_string_new("");
+
+	pthread_cleanup_push(get_reply_cleanup, &data);
 
 	/* Wait for activity on the socket, when there is some,
 	   read all the message line by line */
 	do {
-		bytes = getline(&line, &N, connection->stream);
+		bytes = getline(&data.line, &N, connection->stream);
 		if (bytes == -1) {
 			SPD_DBG
 			    ("Error: Can't read reply, broken socket in get_reply!");
@@ -1788,22 +1817,24 @@ static char *get_reply(SPDConnection * connection)
 			connection->stream = NULL;
 			errors = TRUE;
 		} else {
-			g_string_append(str, line);
+			g_string_append(data.str, data.line);
 		}
 		/* terminate if we reached the last line (without '-' after numcode) */
-	} while (!errors && !((strlen(line) < 4) || (line[3] == ' ')));
+	} while (!errors && !((strlen(data.line) < 4) || (data.line[3] == ' ')));
 
-	free(line);		/* getline allocates with malloc. */
+	pthread_cleanup_pop(0);
+
+	free(data.line);		/* getline allocates with malloc. */
 
 	if (errors) {
 		/* Free the GString and its character data, and return NULL. */
-		g_string_free(str, TRUE);
+		g_string_free(data.str, TRUE);
 		reply = NULL;
 	} else {
 		/* The resulting message received from the socket is stored in reply */
-		reply = str->str;
+		reply = data.str->str;
 		/* Free the GString, but not its character data. */
-		g_string_free(str, FALSE);
+		g_string_free(data.str, FALSE);
 	}
 
 	return reply;
